@@ -1,10 +1,10 @@
 from typing import Optional, TYPE_CHECKING
+from collections import OrderedDict
 
-from PySide2.QtWidgets import QDialog, QVBoxLayout, QHBoxLayout, QLabel, QPushButton, QLineEdit
-from angr.analyses.decompiler.structured_codegen.c import CVariable, CFunction, CConstruct, CFunctionCall
+from PySide2.QtWidgets import QDialog, QVBoxLayout, QHBoxLayout, QLabel, QPushButton, QLineEdit, QListWidget
+from angr.analyses.decompiler.structured_codegen.c import CVariable, CFunction, CConstruct, CFunctionCall, CStructField
 
 if TYPE_CHECKING:
-    from angrmanagement.ui.views.disassembly_view import DisassemblyView
     from angrmanagement.ui.views.code_view import CodeView
 
 
@@ -23,22 +23,21 @@ class NodeNameBox(QLineEdit):
 
     @staticmethod
     def _is_valid_node_name(name):
-        return name and not (' ' in name.strip())
+        return name and not ' ' in name.strip()
 
 
 class RenameNode(QDialog):
-    def __init__(self, disasm_view: Optional['DisassemblyView'] = None, code_view: Optional['CodeView'] = None,
-                 node: Optional[CConstruct] = None, parent=None):
+    def __init__(self, code_view: Optional['CodeView']=None, node: Optional[CConstruct]=None, parent=None):
         super().__init__(parent)
 
         # initialization
-        self._disasm_view = disasm_view
         self._code_view = code_view
         self._node = node
 
-        self._name_box = None
+        self._name_box: NodeNameBox = None
         self._status_label = None
-        self._ok_button = None
+        self._ok_button: QPushButton = None
+        self._suggestion_box: QListWidget = None
 
         self.setWindowTitle('Rename Variable')
 
@@ -46,6 +45,16 @@ class RenameNode(QDialog):
 
         self._init_widgets()
 
+        if not isinstance(self._node, CVariable):
+            self._suggestion_box.setEnabled(False)
+        else:
+            if self._node.unified_variable is not None:
+                if not self._node.unified_variable.candidate_names:
+                    self._suggestion_box.setEnabled(False)
+                else:
+                    for candidate in self._node.unified_variable.candidate_names:
+                        self._suggestion_box.addItem(candidate)
+
         self.setLayout(self.main_layout)
 
     #
@@ -70,6 +79,8 @@ class RenameNode(QDialog):
                 name_box.setText(self._node.name)
             elif isinstance(self._node, CFunctionCall):
                 name_box.setText(self._node.callee_func.name)
+            elif isinstance(self._node, CStructField):
+                name_box.setText(self._node.field)
 
             name_box.selectAll()
         self._name_box = name_box
@@ -79,6 +90,17 @@ class RenameNode(QDialog):
         label_layout.addWidget(name_box)
         self.main_layout.addLayout(label_layout)
 
+        # suggestions
+        suggest_label = QLabel(self)
+        suggest_label.setText("Suggestions")
+
+        suggestion_box = QListWidget()
+        self._suggestion_box = suggestion_box
+        suggestion_layout = QHBoxLayout()
+        suggestion_layout.addWidget(suggest_label)
+        suggestion_layout.addWidget(suggestion_box)
+        self.main_layout.addLayout(suggestion_layout)
+
         # status label
         status_label = QLabel(self)
         self.main_layout.addWidget(status_label)
@@ -128,19 +150,59 @@ class RenameNode(QDialog):
         node_name = self._name_box.name
         if node_name is not None:
             if self._code_view is not None and self._node is not None:
+                # need workspace for altering callbacks of changes
+                workspace = self._code_view.workspace
+                code_kb = self._code_view.codegen.kb
+
                 if isinstance(self._node, CVariable) and self._node.unified_variable is not None:
+                    # callback
+                    # sanity check that we are a stack var
+                    if hasattr(self._node.variable, 'offset') and self._node.variable.offset is not None:
+                        var_type = self._node.type
+                        workspace.plugins.handle_variable_rename(code_kb.functions[self._node.variable.region],
+                                                                 self._node.variable.offset,
+                                                                 self._node.variable.name,
+                                                                 node_name,
+                                                                 var_type,
+                                                                 self._node.variable.size
+                                                                 )
+
                     self._node.unified_variable.name = node_name
                     self._node.unified_variable.renamed = True
                 elif isinstance(self._node, CVariable) and self._node.variable.region == '':
+                    # callback not supported
                     self._code_view.workspace.instance.kb.labels[self._node.variable.addr] = node_name
                     self._node.variable.name = node_name
+                    self._node.variable.renamed = True
+                elif isinstance(self._node, CVariable):
+                    # function argument, probably?
+                    self._node.variable.name = node_name
+                    self._node.variable.renamed = True
                 elif isinstance(self._node, CFunction):
-                    code_kb = self._code_view.codegen.kb
-                    code_kb.functions[self._node.name].name = node_name
+                    # callback
+                    workspace.plugins.handle_function_rename(code_kb.functions.get_by_addr(self._node.addr),
+                                                             self._node.name, node_name)
+
+                    code_kb.functions.get_by_addr(self._node.addr).name = node_name
                     self._node.name = node_name
                     self._node.demangled_name = node_name
                 elif isinstance(self._node, CFunctionCall):
-                    self._node.callee_func.name = node_name
+                    # callback
+                    if self._node.callee_func is not None:
+                        workspace.plugins.handle_function_rename(
+                            code_kb.functions.get_by_addr(self._node.callee_func.addr),
+                            self._node.callee_func.name, node_name
+                        )
+
+                        self._node.callee_func.name = node_name
+                elif isinstance(self._node, CStructField):
+                    # TODO add callback
+                    # TODO prevent name duplication. reuse logic from CTypeEditor?
+                    # TODO if this is a temporary struct, make it permanent and add it to kb.types
+                    fields = [(node_name if n == self._node.field else n, t) for n, t in self._node.type.fields.items()]
+                    self._node.type.fields = OrderedDict(fields)
+                    self._node.field = node_name
+
 
                 self._code_view.codegen.am_event()
                 self.close()
