@@ -1,38 +1,46 @@
+# pylint:disable=no-self-use
 import os
 import logging
 import sys
-from typing import Optional
+import time
+from typing import Optional, TYPE_CHECKING
 
 from PySide2.QtWidgets import QMainWindow, QTabWidget, QFileDialog, QProgressBar
-from PySide2.QtWidgets import QMessageBox, QSplitter, QShortcut
+from PySide2.QtWidgets import QMessageBox, QSplitter, QShortcut, QTabBar
 from PySide2.QtGui import QResizeEvent, QIcon, QDesktopServices, QKeySequence
 from PySide2.QtCore import Qt, QSize, QEvent, QTimer, QUrl
 
 import angr
+import angr.flirt
 try:
     from angr.angrdb import AngrDB
-except ImportError as ex:
-    print(str(ex))
+except ImportError:
     AngrDB = None  # type: Optional[type]
 
 
 try:
     import archr
     import keystone
-except ImportError as e:
+except ImportError:
     archr = None
     keystone = None
 
+from ..daemon import daemon_exists, run_daemon_process, daemon_conn
+from ..daemon.client import ClientService
 from ..logic import GlobalInfo
 from ..data.instance import Instance
+from ..data.library_docs import LibraryDocs
 from ..data.jobs.loading import LoadTargetJob, LoadBinaryJob
 from ..data.jobs import DependencyAnalysisJob
+from ..config import IMG_LOCATION, Conf
+from ..utils.io import isurl, download_url
+from ..utils.env import is_pyinstaller, app_root
+from ..errors import InvalidURLError, UnexpectedStatusCodeError
 from .menus.file_menu import FileMenu
 from .menus.analyze_menu import AnalyzeMenu
 from .menus.help_menu import HelpMenu
 from .menus.view_menu import ViewMenu
 from .menus.plugin_menu import PluginMenu
-from ..config import IMG_LOCATION
 from .workspace import Workspace
 from .dialogs.load_plugins import LoadPlugins
 from .dialogs.load_docker_prompt import LoadDockerPrompt, LoadDockerPromptError
@@ -40,10 +48,9 @@ from .dialogs.new_state import NewState
 from .dialogs.about import LoadAboutDialog
 from .dialogs.preferences import Preferences
 from .toolbars import StatesToolbar, AnalysisToolbar, FileToolbar
-from ..utils.io import isurl, download_url
-from ..errors import InvalidURLError, UnexpectedStatusCodeError
-from ..config import Conf
-from .. import plugins
+
+if TYPE_CHECKING:
+    from PySide2.QtWidgets import QApplication
 
 _l = logging.getLogger(name=__name__)
 
@@ -52,8 +59,8 @@ class MainWindow(QMainWindow):
     """
     The main window of angr management.
     """
-    def __init__(self, parent=None, show=True):
-        super(MainWindow, self).__init__(parent)
+    def __init__(self, app: Optional['QApplication']=None, parent=None, show=True, use_daemon=False):
+        super().__init__(parent)
 
         icon_location = os.path.join(IMG_LOCATION, 'angr.png')
         self.setWindowIcon(QIcon(icon_location))
@@ -64,7 +71,8 @@ class MainWindow(QMainWindow):
         self.setMinimumSize(QSize(400, 400))
         self.setDockNestingEnabled(True)
 
-        self.workspace = None
+        self.app: Optional['QApplication'] = app
+        self.workspace: Workspace = None
         self.central_widget = None
         self.central_widget2 = None
 
@@ -89,9 +97,17 @@ class MainWindow(QMainWindow):
         self._init_toolbars()
         self._init_statusbar()
         self._init_workspace()
-        self._init_shortcuts()
         self._init_menus()
         self._init_plugins()
+        self._init_library_docs()
+        self._init_url_scheme_handler()
+
+        self.workspace.plugins.on_workspace_initialized(self)
+
+        self._init_shortcuts()
+        self._init_flirt_signatures()
+
+        self._run_daemon(use_daemon=use_daemon)
 
         # I'm ready to show off!
         if show:
@@ -101,7 +117,7 @@ class MainWindow(QMainWindow):
 
         self.status = "Ready."
 
-    def sizeHint(self, *args, **kwargs):
+    def sizeHint(self, *args, **kwargs):  # pylint: disable=unused-argument,no-self-use
         return QSize(1200, 800)
 
     #
@@ -123,7 +139,6 @@ class MainWindow(QMainWindow):
     @status.setter
     def status(self, v):
         self._status = v
-
         self.statusBar().showMessage(v)
 
     @property
@@ -141,18 +156,24 @@ class MainWindow(QMainWindow):
     #
 
     def _open_mainfile_dialog(self):
-        file_path, _ = QFileDialog.getOpenFileName(self, "Open a binary", "",
-                                                   "All executables (*);;Windows PE files (*.exe);;Core Dumps (*.core);;angr database (*.adb)",
+        # pylint: disable=assigning-non-slot
+        # https://github.com/PyCQA/pylint/issues/3793
+        file_path, _ = QFileDialog.getOpenFileName(self, "Open a binary", Conf.last_used_directory,
+                                                   "All executables (*);;"
+                                                   "Windows PE files (*.exe);;"
+                                                   "Core Dumps (*.core);;"
+                                                   "angr database (*.adb)",
                                                    )
+        Conf.last_used_directory = os.path.dirname(file_path)
         return file_path
 
     def _pick_image_dialog(self):
         try:
-            prompt = LoadDockerPrompt()
+            prompt = LoadDockerPrompt(parent=self)
         except LoadDockerPromptError:
-            return
+            return None
         if prompt.exec_() == 0:
-            return # User canceled
+            return None  # User canceled
         return prompt.textValue()
 
     def open_load_plugins_dialog(self):
@@ -245,7 +266,7 @@ class MainWindow(QMainWindow):
         self.central_widget.setTabPosition(Qt.RightDockWidgetArea, QTabWidget.North)
         self.central_widget2.setTabPosition(Qt.LeftDockWidgetArea, QTabWidget.North)
 
-        def set_caption(**kwargs):
+        def set_caption(**kwargs):  # pylint: disable=unused-argument
             if self.workspace.instance.project.am_none:
                 self.caption = ''
             elif self.workspace.instance.project.filename is None:
@@ -254,10 +275,15 @@ class MainWindow(QMainWindow):
                 self.caption = os.path.basename(self.workspace.instance.project.filename)
         self.workspace.instance.project.am_subscribe(set_caption)
 
+        self.tab = self.central_widget.findChild(QTabBar)
+        self.tab.tabBarClicked.connect(self.on_center_tab_clicked)
     #
     # Shortcuts
     #
 
+    def interrupt_current_job(self):
+        self.workspace.instance.interrupt_current_job()
+
     def _init_shortcuts(self):
         """
         Initialize shortcuts
@@ -269,6 +295,8 @@ class MainWindow(QMainWindow):
         for i in range(1, len(center_dockable_views)+1):
             QShortcut(QKeySequence('Ctrl+'+str(i)), self, center_dockable_views[i-1].raise_)
 
+        QShortcut(QKeySequence("Ctrl+I"), self, self.interrupt_current_job)
+
         # Raise the DisassemblyView after everything has initialized
         center_dockable_views[0].raise_()
 
@@ -279,14 +307,108 @@ class MainWindow(QMainWindow):
     def _init_plugins(self):
         self.workspace.plugins.discover_and_initialize_plugins()
 
+    #
+    # FLIRT Signatures
+    #
+
+    def _init_flirt_signatures(self):
+        if Conf.flirt_signatures_root:
+            # if it's a relative path, it's relative to the angr-management package
+            if os.path.isabs(Conf.flirt_signatures_root):
+                flirt_signatures_root = Conf.flirt_signatures_root
+            else:
+                if is_pyinstaller():
+                    flirt_signatures_root = os.path.join(app_root(), Conf.flirt_signatures_root)
+                else:
+                    # when running as a Python package, we should use the git submodule, which is on the same level
+                    # with (instead of inside) the angrmanagement module directory.
+                    flirt_signatures_root = os.path.join(app_root(), "..", Conf.flirt_signatures_root)
+            flirt_signatures_root = os.path.normpath(flirt_signatures_root)
+            _l.info("Loading FLIRT signatures from %s.", flirt_signatures_root)
+            angr.flirt.load_signatures(flirt_signatures_root)
+
+    #
+    # Library docs
+    #
+
+    def _init_library_docs(self):
+        GlobalInfo.library_docs = LibraryDocs()
+        if Conf.library_docs_root:
+            GlobalInfo.library_docs.load_func_docs(Conf.library_docs_root)
+
+    #
+    # Daemon
+    #
+
+    def _run_daemon(self, use_daemon=None):
+
+        if use_daemon is None:
+            # Load it from the configuration file
+            use_daemon = Conf.use_daemon
+
+        if not use_daemon:
+            return
+
+        # connect to daemon (if there is one)
+        if not daemon_exists():
+            print("[+] Starting a new daemon.")
+            run_daemon_process()
+            time.sleep(0.2)
+        else:
+            print("[+] Connecting to an existing angr management daemon.")
+
+        while True:
+            try:
+                GlobalInfo.daemon_conn = daemon_conn(service=ClientService)
+            except ConnectionRefusedError:
+                print("[-] Connection failed... try again.")
+                time.sleep(0.4)
+                continue
+            print("[+] Connected to daemon.")
+            break
+
+        from rpyc import BgServingThread
+        _ = BgServingThread(GlobalInfo.daemon_conn)
+
+    #
+    # URL scheme handler setup
+    #
+
+    def _init_url_scheme_handler(self):
+        # URL scheme
+        from ..logic.url_scheme import AngrUrlScheme  # pylint:disable=import-outside-toplevel
+
+        scheme = AngrUrlScheme()
+        registered, _ = scheme.is_url_scheme_registered()
+        supported = scheme.is_url_scheme_supported()
+        checrs_plugin = self.workspace.plugins.get_plugin_instance_by_name("ChessConnector")
+        if checrs_plugin is None:
+            return
+
+        if not registered and supported:
+            btn = QMessageBox.question(None, "Setting up angr URL scheme",
+                                       "angr URL scheme allows \"deep linking\" from browsers and other applications "
+                                       "by registering the angr:// protocol to the current user. Do you want to "
+                                       "register it? You may unregister at any "
+                                       "time in Preferences.",
+                                       defaultButton=QMessageBox.Yes)
+            if btn == QMessageBox.Yes:
+                try:
+                    AngrUrlScheme().register_url_scheme()
+                except (ValueError, FileNotFoundError) as ex:
+                    QMessageBox.warning(None, "Error in registering angr URL scheme",
+                                        "Failed to register the angr URL scheme.\n"
+                                        "The following exception occurred:\n"
+                                        + str(ex))
+
     #
     # Event
     #
 
-    def resizeEvent(self, event):
+    def resizeEvent(self, event: QResizeEvent):
         """
 
-        :param QResizeEvent event:
+        :param event:
         :return:
         """
 
@@ -326,13 +448,13 @@ class MainWindow(QMainWindow):
 
             try:
                 event.result = event.execute()
-            except Exception as e:
-                event.exception = e
+            except Exception as ex:  # pylint:disable=broad-except
+                event.exception = ex
             event.event.set()
 
             return True
 
-        return super(MainWindow, self).event(event)
+        return super().event(event)
 
     def on_screen_changed(self, screen):
         """
@@ -341,6 +463,9 @@ class MainWindow(QMainWindow):
         self.workspace.current_screen.am_obj = screen
         self.workspace.current_screen.am_event()
 
+    def on_center_tab_clicked(self, index):
+        self.workspace.view_manager.handle_center_tab_click(index)
+
     #
     # Actions
     #
@@ -472,10 +597,10 @@ class MainWindow(QMainWindow):
         self.close()
 
     def run_variable_recovery(self):
-        self.workspace.view_manager.first_view_in_category('disassembly').variable_recovery_flavor = 'accurate'
+        self.workspace._get_or_create_disassembly_view().variable_recovery_flavor = 'accurate'
 
     def run_induction_variable_analysis(self):
-        self.workspace.view_manager.first_view_in_category('disassembly').run_induction_variable_analysis()
+        self.workspace._get_or_create_disassembly_view().run_induction_variable_analysis()
 
     def run_dependency_analysis(self, func_addr: Optional[int]=None, func_arg_idx: Optional[int]=None):
         if self.workspace is None or self.workspace.instance is None:
@@ -519,8 +644,11 @@ class MainWindow(QMainWindow):
             return
 
         angrdb = AngrDB()
+        other_kbs = { }
+        extra_info = { }
         try:
-            proj = angrdb.load(file_path)
+            proj = angrdb.load(file_path, kb_names=["global", "pseudocode_variable_kb"], other_kbs=other_kbs,
+                               extra_info=extra_info)
         except angr.errors.AngrIncompatibleDBError as ex:
             QMessageBox.critical(None, 'Error',
                                  "Failed to load the angr database because of compatibility issues.\n"
@@ -542,11 +670,16 @@ class MainWindow(QMainWindow):
         self.workspace.instance.project = proj
         self.workspace.instance.cfg = cfg
         self.workspace.instance.cfb = cfb
+        if "pseudocode_variable_kb" in other_kbs:
+            self.workspace.instance.pseudocode_variable_kb = other_kbs["pseudocode_variable_kb"]
+        else:
+            self.workspace.instance.initialize_pseudocode_variable_kb()
         self.workspace.instance.project.am_event(initialized=True)
 
         # trigger callbacks
         self.workspace.reload()
         self.workspace.on_cfg_generated()
+        self.workspace.plugins.angrdb_load_entries(extra_info)
 
     def _save_database(self, file_path):
         if self.workspace.instance is None or self.workspace.instance.project.am_none:
@@ -557,8 +690,16 @@ class MainWindow(QMainWindow):
                                  'AngrDB is not enabled. Maybe you do not have SQLAlchemy installed?')
             return False
 
+        self.workspace.plugins.handle_project_save(file_path)
+
         angrdb = AngrDB(project=self.workspace.instance.project)
-        angrdb.dump(file_path)
+        extra_info = self.workspace.plugins.angrdb_store_entries()
+        angrdb.dump(file_path, kbs=[
+                        self.workspace.instance.kb,
+                        self.workspace.instance.pseudocode_variable_kb,
+                    ],
+                    extra_info=extra_info,
+                    )
 
         self.workspace.instance.database_path = file_path
         return True
