@@ -12,6 +12,7 @@ class ViewMenu(Menu):
             MenuEntry('Next Tab', main_window.workspace.view_manager.next_tab, shortcut=QKeySequence("Ctrl+Tab")),
             MenuEntry('Previous Tab', main_window.workspace.view_manager.previous_tab, shortcut=QKeySequence("Ctrl+Shift+Tab")),
             MenuSeparator(),
+            MenuEntry('New Disassembly View', main_window.workspace.new_disassembly_view, shortcut=QKeySequence("Ctrl+N")),
             MenuEntry('Split / Unsplit View', main_window.workspace.toggle_split, shortcut=QKeySequence("Ctrl+D")),
             MenuSeparator(),
             MenuEntry('Linear Disassembly', main_window.workspace.show_linear_disassembly_view),
@@ -19,4 +20,10 @@ class ViewMenu(Menu):
             MenuEntry('Symbolic Execution', main_window.workspace.show_symexec_view),
             MenuEntry('Symbolic States', main_window.workspace.show_states_view),
             MenuEntry('Strings', main_window.workspace.show_strings_view),
+            MenuEntry('Proximity View', main_window.view_proximity_for_current_function),
+            MenuEntry('Patches', main_window.workspace.show_patches_view),
+            MenuEntry('Interaction', main_window.workspace.show_interaction_view),
+            MenuEntry('Types', main_window.workspace.show_types_view),
+            MenuEntry('Functions', main_window.workspace.show_functions_view),
+            MenuEntry('Console', main_window.workspace.show_console_view),
         ])
