@@ -1,39 +1,55 @@
+
+from collections import defaultdict
 import logging
 from typing import Union, Optional, TYPE_CHECKING
 
-from PySide2.QtWidgets import QHBoxLayout, QVBoxLayout, QApplication, QMessageBox
-from PySide2.QtCore import Qt, QSize, Signal
+from PySide2.QtWidgets import QHBoxLayout, QVBoxLayout, QApplication, QMessageBox, QMenu, QAction
+from PySide2.QtCore import Qt, Signal
+from PySide2.QtGui import QCursor
 
+from ...logic import GlobalInfo
 from ...data.instance import ObjectContainer
 from ...utils import locate_function
 from ...data.function_graph import FunctionGraph
 from ...logic.disassembly import JumpHistory, InfoDock
-from ..widgets import QDisassemblyGraph, QDisasmStatusBar, QLinearDisassembly, QFeatureMap, QLinearDisassemblyView
+from ..widgets import QDisassemblyGraph, QDisasmStatusBar, QLinearDisassembly, QFeatureMap,\
+    QLinearDisassemblyView, DisassemblyLevel
 from ..dialogs.dependson import DependsOn
 from ..dialogs.jumpto import JumpTo
+from ..dialogs.rename import RenameDialog
 from ..dialogs.rename_label import RenameLabel
 from ..dialogs.set_comment import SetComment
 from ..dialogs.new_state import NewState
 from ..dialogs.xref import XRef
+from ..dialogs.hook import HookDialog
+from ..dialogs.func_doc import FuncDocDialog
 from ..menus.disasm_insn_context_menu import DisasmInsnContextMenu
 from ..menus.disasm_label_context_menu import DisasmLabelContextMenu
 from .view import BaseView
+from ..widgets import QFindAddrAnnotation, QAvoidAddrAnnotation, QBlockAnnotations
+from ..widgets.qblock_code import QVariableObj
+from ...ui.widgets.qinst_annotation import QHookAnnotation
 
 if TYPE_CHECKING:
-    from angr.knowledge_plugins import Function
+    from angr.knowledge_plugins import VariableManager
 
 
 _l = logging.getLogger(__name__)
 
 
 class DisassemblyView(BaseView):
+    """
+    Disassembly View
+    """
     view_visibility_changed = Signal()
+    disassembly_level_changed = Signal(DisassemblyLevel)
 
     def __init__(self, workspace, *args, **kwargs):
         super().__init__('disassembly', workspace, *args, **kwargs)
 
-        self.caption = 'Disassembly'
-
+        self.base_caption = 'Disassembly'
+        self._disassembly_level = DisassemblyLevel.MachineCode
+        self._show_minimap: bool = True
         self._show_address = True
         self._show_variable = True
         # whether we want to show identifier or not
@@ -44,18 +60,24 @@ class DisassemblyView(BaseView):
         self._linear_viewer = None  # type: Optional[QLinearDisassembly]
         self._flow_graph = None  # type: Optional[QDisassemblyGraph]
         self._statusbar = None
-        self._jump_history = JumpHistory()
+        self.jump_history: JumpHistory = JumpHistory()
         self.infodock = InfoDock(self)
         self._variable_recovery_flavor = 'fast'
         self.variable_manager = None  # type: Optional[VariableManager]
         self._current_function = ObjectContainer(None, 'The currently selected function')
 
+        # For tests only
+        self._t_linear_viewer_visible: bool = False
+        self._t_flow_graph_visible: bool = False
+
         self._insn_menu = None  # type: Optional[DisasmInsnContextMenu]
         self._label_menu = None  # type: Optional[DisasmLabelContextMenu]
 
         self._insn_addr_on_context_menu = None
         self._label_addr_on_context_menu = None
 
+        self._annotation_callbacks = []
+
         self.width_hint = 800
         self.height_hint = 800
 
@@ -63,6 +85,17 @@ class DisassemblyView(BaseView):
         self._init_menus()
         self._register_events()
 
+    @property
+    def disassembly_level(self):
+        return self._disassembly_level
+
+    def set_disassembly_level(self, level:DisassemblyLevel):
+        self._disassembly_level = level
+        self._flow_graph.set_disassembly_level(level)
+        self._linear_viewer.set_disassembly_level(level)
+        self.disassembly_level_changed.emit(level)
+        self.redraw_current_graph()
+
     def reload(self):
 
         old_infodock = self.infodock.copy()
@@ -88,10 +121,18 @@ class DisassemblyView(BaseView):
     # Properties
     #
 
+    @property
+    def caption(self):
+        return f'{self.base_caption}-{self.index}'
+
     @property
     def disasm(self):
         return self._flow_graph.disasm
 
+    @property
+    def show_minimap(self):
+        return self._show_minimap
+
     @property
     def smart_highlighting(self):
         if self._flow_graph is None:
@@ -134,7 +175,7 @@ class DisassemblyView(BaseView):
 
         :return:    Linear viewer or flow graph.
         """
-        if self._linear_viewer.isVisible():
+        if self._linear_viewer.isVisible() or self._t_linear_viewer_visible:
             return self._linear_viewer
         else:
             return self._flow_graph
@@ -249,6 +290,40 @@ class DisassemblyView(BaseView):
         self._label_menu.qmenu().exec_(pos)
         self._label_addr_on_context_menu = None
 
+    def rename_selected_object(self):
+        """
+        Opens dialog for renaming the currently selected QBlockCodeObj
+        """
+        obj = self.infodock.selected_qblock_code_obj
+        if isinstance(obj, QVariableObj):
+            dlg = RenameDialog('Rename Variable', obj.obj.name, self)
+            dlg.exec_()
+            if dlg.result is not None:
+                obj.obj.name = dlg.result
+                self.current_graph.refresh()
+
+    def get_context_menu_for_selected_object(self) -> Optional[QMenu]:
+        """
+        Returns a QMenu object for the currently selected QBlockCodeObj
+        """
+        obj = self.infodock.selected_qblock_code_obj
+        if isinstance(obj, QVariableObj):
+            rename_act = QAction('Re&name', self)
+            rename_act.triggered.connect(self.rename_selected_object)
+            mnu = QMenu()
+            mnu.addActions([rename_act])
+            return mnu
+        else:
+            return None
+
+    def show_context_menu_for_selected_object(self):
+        """
+        Spawns a context menu for the currently selected QBlockCodeObj
+        """
+        mnu = self.get_context_menu_for_selected_object()
+        if mnu is not None:
+            mnu.exec_(QCursor.pos())
+
     def popup_jumpto_dialog(self):
         JumpTo(self, parent=self).exec_()
 
@@ -279,6 +354,47 @@ class DisassemblyView(BaseView):
         else:
             dialog.exec_()
 
+    def popup_hook_dialog(self, async_=True, addr=None):
+        addr = addr or self._address_in_selection()
+
+        if addr is None:
+            return
+
+        dialog = HookDialog(self.workspace.instance, addr=addr, parent=self)
+        if async_:
+            dialog.show()
+        else:
+            dialog.exec_()
+
+    def popup_func_doc_dialog(self, instr_addr):
+        """
+        Spawns a popup dialog for the currently selected call instruction func_docs
+        """
+        if self._flow_graph is None:
+            return
+        block = self._flow_graph._insaddr_to_block.get(instr_addr, None)
+        if block:
+            instr = block.addr_to_insns[instr_addr]
+            if instr is None or instr.insn.type != "call":
+                return
+            out_targets = instr.out_branch.targets
+            if len(out_targets) != 1:
+                return
+
+            target = next(iter(out_targets))
+            operand = instr.get_operand(0)
+
+            doc_tuple = GlobalInfo.library_docs.get_docstring_for_func_name(operand.text)
+            if doc_tuple is None:
+                doc_string = f"Cannot find local documentation for function {operand.text}."
+                url = "http://"
+                ftype = "<>"
+                doc_tuple = (doc_string, url, ftype)
+            dialog = FuncDocDialog(self.workspace.instance,
+                                   addr=target, name=operand.text, doc_tuple=doc_tuple,
+                                   parent=self)
+            dialog.show()
+
     def popup_dependson_dialog(self, addr: Optional[int]=None, use_operand=False, func: bool=False):
         if use_operand:
             r = self._flow_graph.get_selected_operand_info()
@@ -372,6 +488,7 @@ class DisassemblyView(BaseView):
 
         self._flow_graph.setFocus()
         self.view_visibility_changed.emit()
+        self._flow_graph.refresh()
 
     def display_linear_viewer(self):
 
@@ -386,10 +503,11 @@ class DisassemblyView(BaseView):
 
         self._linear_viewer.setFocus()
         self.view_visibility_changed.emit()
+        self._linear_viewer.refresh()
 
     def display_function(self, function):
 
-        self._jump_history.jump_to(function.addr)
+        self.jump_history.jump_to(function.addr)
         self._display_function(function)
 
     def decompile_current_function(self):
@@ -402,6 +520,13 @@ class DisassemblyView(BaseView):
 
             self.workspace.decompile_function(self._current_function.am_obj, curr_ins=curr_ins)
 
+    def toggle_show_minimap(self, show:bool):
+        """
+        Toggle minimap display preference
+        """
+        self._show_minimap = show
+        self.current_graph.refresh()
+
     def toggle_smart_highlighting(self, enabled):
         """
         Toggle between the smart highlighting mode and the text-based highlighting mode.
@@ -473,20 +598,25 @@ class DisassemblyView(BaseView):
 
         # Record the current instruction address first
         if src_ins_addr is not None:
-            self._jump_history.record_address(src_ins_addr)
+            self.jump_history.record_address(src_ins_addr)
 
-        self._jump_history.jump_to(addr)
+        self.jump_history.jump_to(addr)
         self._jump_to(addr, use_animation=use_animation)
 
         return True
 
     def jump_back(self):
-        addr = self._jump_history.backtrack()
+        addr = self.jump_history.backtrack()
         if addr is not None:
             self._jump_to(addr, use_animation=False)
 
     def jump_forward(self):
-        addr = self._jump_history.forwardstep()
+        addr = self.jump_history.forwardstep()
+        if addr is not None:
+            self._jump_to(addr, use_animation=False)
+
+    def jump_to_history_position(self, pos:int):
+        addr = self.jump_history.step_position(pos)
         if addr is not None:
             self._jump_to(addr, use_animation=False)
 
@@ -515,17 +645,37 @@ class DisassemblyView(BaseView):
             self._flow_graph.update_label(addr, is_renaming=is_renaming)
 
     def avoid_addr_in_exec(self, addr):
+        self.workspace._get_or_create_symexec_view().avoid_addr_in_exec(addr)
 
-        self.workspace.view_manager.first_view_in_category('symexec').avoid_addr_in_exec(addr)
+    def find_addr_in_exec(self, addr):
+        self.workspace._get_or_create_symexec_view().find_addr_in_exec(addr)
 
     def run_induction_variable_analysis(self):
         if self._flow_graph.induction_variable_analysis:
             self._flow_graph.induction_variable_analysis = None
         else:
-            ana = self.workspace.instance.project.analyses.AffineRelationAnalysis(self._flow_graph._function_graph.function)
+            analyses = self.workspace.instance.project.analyses
+            ana = analyses.AffineRelationAnalysis(self._flow_graph._function_graph.function)
             self._flow_graph.induction_variable_analysis = ana
         self._flow_graph.refresh()
 
+    def fetch_qblock_annotations(self, qblock):
+        addr_to_annotations = defaultdict(list)
+        for annotations in self.workspace.plugins.build_qblock_annotations(qblock):
+            addr_to_annotations[annotations.addr].append(annotations)
+        for addr in qblock.addr_to_insns.keys():
+            if addr in self.workspace.instance.project._sim_procedures:
+                hook_annotation = QHookAnnotation(addr)
+                addr_to_annotations[addr].append(hook_annotation)
+            view = self.workspace.view_manager.first_view_in_category("symexec")
+            if view is not None:
+                qsimgrs = view._simgrs
+                if addr in qsimgrs.find_addrs:
+                    addr_to_annotations[addr].append(QFindAddrAnnotation(addr, qsimgrs))
+                if addr in qsimgrs.avoid_addrs:
+                    addr_to_annotations[addr].append(QAvoidAddrAnnotation(addr, qsimgrs))
+        return QBlockAnnotations(addr_to_annotations, parent=qblock, disasm_view=self)
+
     #
     # Initialization
     #
@@ -576,6 +726,8 @@ class DisassemblyView(BaseView):
 
         self.workspace.current_screen.am_subscribe(self.on_screen_changed)
 
+        self.infodock.qblock_code_obj_selection_changed.connect(self.redraw_current_graph)
+
     #
     # Private methods
     #
@@ -595,20 +747,22 @@ class DisassemblyView(BaseView):
         # clear existing selected instructions and operands
         self.infodock.clear_selection()
 
-        if self._flow_graph.isVisible():
+        if self._flow_graph.isVisible() or self._t_flow_graph_visible:
             if self._flow_graph.function_graph is None or self._flow_graph.function_graph.function is not the_func:
                 # set function graph of a new function
                 self._flow_graph.function_graph = FunctionGraph(function=the_func,
                                                                 exception_edges=self.show_exception_edges,
                                                                 )
 
-        elif self._linear_viewer.isVisible():
+        elif self._linear_viewer.isVisible() or self._t_linear_viewer_visible:
             self._linear_viewer.navigate_to_addr(the_func.addr)
 
-        self.workspace.view_manager.first_view_in_category('console').push_namespace({
-            'func': the_func,
-            'function_': the_func,
-        })
+        view = self.workspace.view_manager.first_view_in_category('console')
+        if view is not None:
+            view.push_namespace({
+                'func': the_func,
+                'function_': the_func,
+            })
 
     def _jump_to(self, addr, use_animation=False):
         function = locate_function(self.workspace.instance, addr)
