@@ -1,36 +1,44 @@
-from typing import TYPE_CHECKING, Optional
+from typing import TYPE_CHECKING, Callable
 import logging
 import traceback
+from string import ascii_uppercase
 
 from PySide2.QtCore import Qt, QSettings
 
+from angr.knowledge_plugins.functions.function import Function as angrFunc
 from angr.knowledge_plugins import Function
 from angr import StateHierarchy
 
 from ..config import Conf
 from ..data.instance import ObjectContainer
-from ..data.jobs import CodeTaggingJob, PrototypeFindingJob, VariableRecoveryJob
+from ..data.jobs import CodeTaggingJob, PrototypeFindingJob, VariableRecoveryJob, FlirtSignatureRecognitionJob
 from .views import (FunctionsView, DisassemblyView, SymexecView, StatesView, StringsView, ConsoleView, CodeView,
-                    InteractionView, PatchesView, DependencyView, ProximityView)
+                    InteractionView, PatchesView, DependencyView, ProximityView, TypesView)
 from .widgets.qsmart_dockwidget import QSmartDockWidget
 from .view_manager import ViewManager
+from .menus.disasm_insn_context_menu import DisasmInsnContextMenu
 
 from ..plugins import PluginManager
 
 if TYPE_CHECKING:
     from ..data.instance import Instance
+    from angrmanagement.ui.main_window import MainWindow
 
 
 _l = logging.getLogger(__name__)
 
 
 class Workspace:
+    """
+    This class implements the angr management workspace.
+    """
     def __init__(self, main_window, instance):
 
-        self._main_window = main_window
+        self.main_window: 'MainWindow' = main_window
         self._instance = instance
         self.is_split = False
         self.split_tab_id = 0
+        self.last_unsplit_view = None
         instance.workspace = self
 
         self.view_manager: ViewManager = ViewManager(self)
@@ -75,6 +83,10 @@ class Workspace:
     # Properties
     #
 
+    @property
+    def _main_window(self) -> 'MainWindow':
+        return self.main_window
+
     @property
     def instance(self) -> 'Instance':
         return self._instance
@@ -94,8 +106,8 @@ class Workspace:
     def on_cfg_generated(self):
 
         self.instance.add_job(
-            PrototypeFindingJob(
-                on_finish=self._on_prototype_found,
+            FlirtSignatureRecognitionJob(
+                on_finish=self._on_flirt_signature_recognized,
             )
         )
 
@@ -109,18 +121,35 @@ class Workspace:
                 self.on_function_selected(the_func)
 
             # Initialize the linear viewer
-            self.view_manager.first_view_in_category('disassembly')._linear_viewer.initialize()
+            if len(self.view_manager.views_by_category['disassembly']) == 1:
+                view = self.view_manager.first_view_in_category('disassembly')
+            else:
+                view = self.view_manager.current_view_in_category('disassembly')
+            if view is not None:
+                view._linear_viewer.initialize()
 
             # Reload the pseudocode view
-            self.view_manager.first_view_in_category('pseudocode').reload()
+            view = self.view_manager.first_view_in_category('pseudocode')
+            if view is not None:
+                view.reload()
 
             # Reload the strings view
-            self.view_manager.first_view_in_category('strings').reload()
+            view = self.view_manager.first_view_in_category('strings')
+            if view is not None:
+                view.reload()
+
+    def _on_flirt_signature_recognized(self):
+        self.instance.add_job(
+            PrototypeFindingJob(
+                on_finish=self._on_prototype_found,
+            )
+        )
 
     def _on_prototype_found(self):
         self.instance.add_job(
             VariableRecoveryJob(
                 on_finish=self.on_variable_recovered,
+                **self.instance.variable_recovery_args,
             )
         )
 
@@ -133,7 +162,11 @@ class Workspace:
 
     def on_function_tagged(self):
         # reload disassembly view
-        view = self.view_manager.first_view_in_category('disassembly')
+        if len(self.view_manager.views_by_category['disassembly']) == 1:
+            view = self.view_manager.first_view_in_category('disassembly')
+        else:
+            view = self.view_manager.current_view_in_category('disassembly')
+
         if view is not None:
             view: DisassemblyView
             if view.current_function.am_obj is not None:
@@ -143,6 +176,29 @@ class Workspace:
     # Public methods
     #
 
+    def new_disassembly_view(self):
+        """
+        Add a new disassembly view into the
+        central_widget with a unique id
+
+        :return:    None
+        """
+
+        dis_views = self.view_manager.views_by_category['disassembly']
+        dis_ids = [view.index for view in dis_views]
+        dis_ids.sort()
+        missing_ids = sorted(set(range(dis_ids[0], dis_ids[-1])) - set(dis_ids))
+        new_view = DisassemblyView(self, 'center')
+        if missing_ids:
+            new_view.index = missing_ids[0]
+        else:
+            new_view.index = dis_ids[-1]+1
+        self.add_view(new_view, new_view.caption, new_view.category)
+        self.raise_view(new_view)
+        if self.instance.binary_path is not None:
+            self.on_cfg_generated()
+        # TODO move new_view tab to front of dock
+
     def split_view(self):
         """
         Split the view into two panes and shift
@@ -151,7 +207,7 @@ class Workspace:
         :return:    None
         """
 
-        window_id = self.view_manager.get_current_tab_id()
+        window_id = self.view_manager.get_current_tab_id() + 1
         if self.is_split is False:
             self._main_window.central_widget.removeDockWidget(self.view_manager.docks[window_id])
             dock_area = ViewManager.DOCKING_POSITIONS.get(self.default_tabs[window_id].default_docking_position,
@@ -226,9 +282,8 @@ class Workspace:
         for view in views:
             try:
                 view.reload()
-            except Exception:
+            except Exception:  # pylint:disable=broad-except
                 _l.warning("Exception occurred during reloading view %s.", view, exc_info=True)
-                pass
 
     def viz(self, obj):
         """
@@ -249,7 +304,7 @@ class Workspace:
             self.jump_to(obj.addr)
 
     def jump_to(self, addr, view=None, use_animation=False):
-        if view is None or view.category != "disassembly":
+        if view is None or view.category != 'disassembly':
             view = self._get_or_create_disassembly_view()
 
         view.jump_to(addr, use_animation=use_animation)
@@ -257,13 +312,19 @@ class Workspace:
         view.setFocus()
 
     def set_comment(self, addr, comment_text):
-
         kb = self.instance.project.kb
-        if comment_text is None and addr in kb.comments:
+        exists = addr in kb.comments
+
+        # callback
+        if comment_text is None and exists:
+            self.plugins.handle_comment_changed(addr, "", False, False)
             del kb.comments[addr]
-        kb.comments[addr] = comment_text
+        else:
+            self.plugins.handle_comment_changed(addr, comment_text, not exists, False)
+            kb.comments[addr] = comment_text
 
         # callback first
+        # TODO: can this be removed?
         if self.instance.set_comment_callback:
             self.instance.set_comment_callback(addr=addr, comment_text=comment_text)
 
@@ -308,6 +369,7 @@ class Workspace:
         view.function.am_obj = func
         view.function.am_event(focus=True, focus_addr=curr_ins)
 
+
     def create_simulation_manager(self, state, state_name, view=None):
 
         inst = self.instance
@@ -369,13 +431,41 @@ class Workspace:
         self.raise_view(view)
         view.setFocus()
 
+    def show_patches_view(self):
+        view = self._get_or_create_patches_view()
+        self.raise_view(view)
+        view.setFocus()
+
+    def show_interaction_view(self):
+        view = self._get_or_create_interaction_view()
+        self.raise_view(view)
+        view.setFocus()
+
+    def show_types_view(self):
+        view = self._get_or_create_types_view()
+        self.raise_view(view)
+        view.setFocus()
+
+    def show_functions_view(self):
+        view = self._get_or_create_functions_view()
+        self.raise_view(view)
+        view.setFocus()
+
+    def show_console_view(self):
+        view = self._get_or_create_console_view()
+        self.raise_view(view)
+        view.setFocus()
+
     #
     # Private methods
     #
 
     def _get_or_create_disassembly_view(self) -> DisassemblyView:
         # Take the first disassembly view
-        view = self.view_manager.first_view_in_category("disassembly")
+        if len(self.view_manager.views_by_category['disassembly']) == 1:
+            view = self.view_manager.first_view_in_category('disassembly')
+        else:
+            view = self.view_manager.current_view_in_category('disassembly')
 
         if view is None:
             # Create a new disassembly view
@@ -429,6 +519,17 @@ class Workspace:
 
         return view
 
+    def _get_or_create_patches_view(self):
+        # Take the first strings view
+        view = self.view_manager.first_view_in_category("patches")
+
+        if view is None:
+            # Create a new states view
+            view = PatchesView(self, 'center')
+            self.add_view(view, view.caption, view.category)
+
+        return view
+
     def _get_or_create_interaction_view(self):
         view = self.view_manager.first_view_in_category("interaction")
         if view is None:
@@ -437,6 +538,14 @@ class Workspace:
             self.add_view(view, view.caption, view.category)
         return view
 
+    def _get_or_create_types_view(self):
+        view = self.view_manager.first_view_in_category("types")
+        if view is None:
+            # Create a new interaction view
+            view = TypesView(self, 'center')
+            self.add_view(view, view.caption, view.category)
+        return view
+
     def _get_or_create_proximity_view(self) -> ProximityView:
         # Take the first proximity view
         view = self.view_manager.first_view_in_category("proximity")
@@ -448,42 +557,75 @@ class Workspace:
 
         return view
 
+    def _get_or_create_console_view(self) -> ConsoleView:
+        # Take the first console view
+        view = self.view_manager.first_view_in_category("console")
+
+        if view is None:
+            # Create a new console view
+            view = ConsoleView(self, 'bottom')
+            self.add_view(view, view.caption, view.category)
+
+        return view
+
+    def _get_or_create_functions_view(self) -> FunctionsView:
+        # Take the first functions view
+        view = self.view_manager.first_view_in_category("functions")
+
+        if view is None:
+            # Create a new functions view
+            view = FunctionsView(self, 'left')
+            self.add_view(view, view.caption, view.category)
+
+        return view
+
     #
     # UI-related Callback Setters & Manipulation
     #
 
     # TODO: should these be removed? Nobody is using them and there is equivalent functionality elsewhere.
 
-    from typing import Callable
-    from angr.knowledge_plugins.functions.function import Function as angrFunc
-    from .menus.disasm_insn_context_menu import DisasmInsnContextMenu
-
     def set_cb_function_backcolor(self, callback: Callable[[angrFunc], None]):
         fv = self.view_manager.first_view_in_category('functions')  # type: FunctionsView
         if fv:
             fv.backcolor_callback = callback
 
     def set_cb_insn_backcolor(self, callback: Callable[[int, bool], None]):
-        dv = self.view_manager.first_view_in_category('disassembly')  # type: DisassemblyView
+        if len(self.view_manager.views_by_category['disassembly']) == 1:
+            dv = self.view_manager.first_view_in_category('disassembly')  # type: DisassemblyView
+        else:
+            dv = self.view_manager.current_view_in_category('disassembly')  # type: DisassemblyView
         if dv:
             dv.insn_backcolor_callback = callback
 
     def set_cb_label_rename(self, callback):
-        dv = self.view_manager.first_view_in_category('disassembly')  # type: DisassemblyView
+        if len(self.view_manager.views_by_category['disassembly']) == 1:
+            dv = self.view_manager.first_view_in_category('disassembly')  # type: DisassemblyView
+        else:
+            dv = self.view_manager.current_view_in_category('disassembly')  # type: DisassemblyView
         if dv:
             dv.label_rename_callback = callback
 
     def add_disasm_insn_ctx_menu_entry(self, text, callback: Callable[[DisasmInsnContextMenu], None], add_separator_first=True):
-        dv = self.view_manager.first_view_in_category('disassembly')  # type: DisassemblyView
+        if len(self.view_manager.views_by_category['disassembly']) == 1:
+            dv = self.view_manager.first_view_in_category('disassembly')  # type: DisassemblyView
+        else:
+            dv = self.view_manager.current_view_in_category('disassembly')  # type: DisassemblyView
         if dv._insn_menu:
             dv._insn_menu.add_menu_entry(text, callback, add_separator_first)
 
     def remove_disasm_insn_ctx_menu_entry(self, text, remove_preceding_separator=True):
-        dv = self.view_manager.first_view_in_category('disassembly')  # type: DisassemblyView
+        if len(self.view_manager.views_by_category['disassembly']) == 1:
+            dv = self.view_manager.first_view_in_category('disassembly')  # type: DisassemblyView
+        else:
+            dv = self.view_manager.current_view_in_category('disassembly')  # type: DisassemblyView
         if dv._insn_menu:
             dv._insn_menu.remove_menu_entry(text, remove_preceding_separator)
 
     def set_cb_set_comment(self, callback):
-        dv = self.view_manager.first_view_in_category('disassembly')  # type: DisassemblyView
+        if len(self.view_manager.views_by_category['disassembly']) == 1:
+            dv = self.view_manager.first_view_in_category('disassembly')  # type: DisassemblyView
+        else:
+            dv = self.view_manager.current_view_in_category('disassembly')  # type: DisassemblyView
         if dv:
             dv.set_comment_callback = callback
